<?php
/**
 * @package BWDPG Blocks
 * Blocks Loader
 */

// Exit if accessed directly.
if ( !defined( 'ABSPATH' ) ) {
    exit;
}

/**
 * Blocks Loader Class
 */
class BWDPG_POST_GRID_BLOCKS_LOADER {

    /**
     * Constructor
     */
    public function __construct() {
        
         // Register Blocks
        add_action( 'init', [ $this, 'register_blocks' ] );
        
        //Register Block Category
        if ( version_compare( BWDPG_POST_GRID_WP_VERSION, '5.8', '>=' ) ) {
            add_filter( 'block_categories_all', [ $this, 'register_post_grid_block_category' ], 99999, 2 );
        } else {
            add_filter( 'block_categories', [ $this, 'register_post_grid_block_category' ], 99999, 2 );
        }

        // Enqueue Inline style on render block
        add_filter( 'render_block', [ $this, 'generate_inline_style_on_render_block' ], 10, 2 );

        // enqueue editor assets
        add_action( 'enqueue_block_editor_assets', [ $this, 'enqueue_editor_assets' ] );

        // enqueue assets for frontend
        add_action( 'enqueue_block_assets', [ $this, 'enqueue_assets' ] );
    }

    /**
     * Enqueue Editor Assets
     */
    public function enqueue_editor_assets(){
        wp_enqueue_script(
            'bwdpg-blocks-global-js',
            BWDPG_POST_GRID_ADMIN_URL . '/dist/global.js',
            [],
            BWDPG_POST_GRID_VERSION,
            true
        );
    }

    /**
     * Enqueue Assets
     */
     public function enqueue_assets(){
         wp_enqueue_script( 'bwdpg-blocks-frontend', BWDPG_POST_GRID_ADMIN_URL . './includes/assets/js/main.js', [], BWDPG_POST_GRID_VERSION, true);
    }
    /**
     * Blocks Category
     */
    public function register_post_grid_block_category( $categories, $post ) {
        return array_merge(
            [
                [
                    'slug' => 'bwdpg-advanced-post-blocks',
                    'title' => __( 'BWD Advanced Post Block', 'bwdpg-post-grid' )
                ],
            ],
            $categories
        );
    }

    /**
     * Load Blocks
     */
    public function register_blocks() {
        
        // get all blocks from includes/blocks/blocks.php
        require_once BWDPG_POST_GRID_DIR_PATH . './includes/blocks/blocks.php';

        // Register Blocks
        if( ! empty( $bwdpg_post_grid_blocks ) && is_array( $bwdpg_post_grid_blocks ) ) {
            foreach( $bwdpg_post_grid_blocks as $block ) {
                $this->register_single_block( $block );
            }
        }

    }

    /**
     * Register Single Block
     */
    public function register_single_block( $block ) {
        register_block_type(
            BWDPG_POST_GRID_DIR_PATH . './build/blocks/' . $block['name'],
            [
                'render_callback' => [ $this, 'bwdpg_post_grid_render_callback']
            ]
        );
        
    }
  
    /**
     * Render callback for the BWD Advanced Post Block.
     */
        public function bwdpg_post_grid_render_callback ($attributes, $content){
            $selectedCategory = isset($attributes['selectedCategory']) ? $attributes['selectedCategory'] : [];
            $categoryIds = array_column($selectedCategory, 'value'); 
            $selectedTag = isset($attributes['selectedTag']) ? $attributes['selectedTag'] : [];
            $tagIds = array_column($selectedTag, 'value'); 
            $selectedAuthor = isset($attributes['selectedAuthor']) ? $attributes['selectedAuthor'] : [];
            $authorIds = array_column($selectedAuthor, 'value');
            $selectCustomPost = isset($attributes['includePosts']) ? $attributes['includePosts'] : [];
            $includeId = array_column($selectCustomPost, 'value');
            $removePost = isset($attributes['excludePosts']) ? $attributes['excludePosts'] : [];
            $excludeId = array_column($removePost, 'value');
            
            // Building the query arguments based on attributes
            $args = [
                'post_type' => 'post',
                "posts_per_page" => isset($attributes['numberOfPost']) ? $attributes['numberOfPost'] : 4,
                'category__in' => $categoryIds,
                "order" => isset($attributes['order']) ? $attributes['order'] : '',
                "orderby" => isset($attributes['orderBy']) ? $attributes['orderBy'] : '',
                'author__in' => $authorIds,
                'tag__in' => $tagIds,
                'post__in' => $includeId,
                'post__not_in' => $excludeId,
                'paged' => get_query_var('paged') ? get_query_var('paged') : 1, 
            ];
            // Creating a new WP_Query instance
            $query = new WP_Query($args);

             // Extracting additional attributes
            $style = isset( $attributes['style'] ) ? $attributes['style'] : 'style-1';
            $container = isset( $attributes['containerWidth'] ) ? $attributes['containerWidth'] : 'container';
            $author_prefix = isset( $attributes['authorPrefix'] ) ? $attributes['authorPrefix'] : 'Written By';
            $read_time_suffix = isset( $attributes['readTimeSuffix'] ) ? $attributes['readTimeSuffix'] : 'Min Read';
            $show_header = isset( $attributes['showHeading'] ) ? $attributes['showHeading'] : false;
            $show_sub_header = isset( $attributes['showSubHeading'] ) ? $attributes['showSubHeading'] : false;
            $show_separator = isset( $attributes['showSeparator'] ) ? $attributes['showSeparator'] : false;
            $show_img_overlay = isset( $attributes['showImageOverlay'] ) ? $attributes['showImageOverlay'] : false;
            $header_title = isset( $attributes['headerTitle'] ) ? $attributes['headerTitle'] : 'Header Title Here';
            $sub_heading = isset( $attributes['subHeading'] ) ? $attributes['subHeading'] : 'Sub Title Here';
            $show_featured_image = isset( $attributes['showFeaturedImage'] ) ? $attributes['showFeaturedImage'] : true;
            $show_title = isset( $attributes['showTitle'] ) ? $attributes['showTitle'] : true;
            $title_tag = isset( $attributes['titleTag'] ) ? $attributes['titleTag'] : 'h2';
            $show_description = isset( $attributes['showDescription'] ) ? $attributes['showDescription'] : true;
            $show_read_more = isset( $attributes['showReadMore'] ) ? $attributes['showReadMore'] : true;
            $show_date = isset( $attributes['showDate'] ) ? $attributes['showDate'] : true;
            $show_category = isset( $attributes['showCategory'] ) ? $attributes['showCategory'] : true;
            $excerpt_length = isset($attributes['wordsExcerpt']) ? absint($attributes['wordsExcerpt']) : 10;
            $title_length = isset($attributes['titleLength']) ? absint($attributes['titleLength']) : 5;
            $tail_indicator = isset( $attributes['wordsTailIndicator'] ) ? $attributes['wordsTailIndicator'] : '';
            $author_icon = isset( $attributes['authorIcon'] ) ? $attributes['authorIcon'] : 'gravatar';
            $show_pagination = isset( $attributes['showPagination'] ) ? $attributes['showPagination'] : false;
            $pagi_prev_text = isset( $attributes['pagiPrevText'] ) ? $attributes['pagiPrevText'] : 'Previous';
            $pagi_next_text = isset( $attributes['pagiNextText'] ) ? $attributes['pagiNextText'] : 'Next';
            $read_more_text = isset( $attributes['readMoreText'] ) ? $attributes['readMoreText'] : 'Read More';
            $item_animation = isset( $attributes['animation'] ) ? $attributes['animation'] : '';
            $customCss = isset($attributes['customCSS']) ? $attributes['customCSS'] : '';
            $custom_id = isset($attributes['id']) ? $attributes['id'] : '';
            $date_format = isset( $attributes['customDate'] ) ? $attributes['customDate'] : 'F j, Y';

            // Get block wrapper attributes
            $blocks_props = get_block_wrapper_attributes([
                'class' => $attributes['uniqueId'] . ' ' . $style . ' ' . $container . ' bwdpg-responsive',
                
            ]);

             // Start output buffering
            ob_start();

            // Ensure that the custom CSS is not empty
            if (!empty($customCss)) {
                    echo '<style type="text/css">' . esc_html($customCss) . '</style>';
            }
           
            // Building the main markup
            $markup = '<div '.$blocks_props.'>';
            // Post Header
             if ($show_header) {
                $markup .= '<div class="bwdpg-header-title">';
                $markup .= '<h2>' . esc_html($header_title) . '</h2>';
                if ($show_sub_header) {
                    $markup .= '<p>' . esc_html($sub_heading) . '</p>';
                }

                if ($show_separator) {
                    $markup .= '<div></div>';
                }

                $markup .= '</div>';
            }
            $markup .= '<div class="bwdpg-item-wrapper ' . esc_attr($item_animation) . '"' . ($custom_id ? ' id="' . esc_attr($custom_id) . '"' : '') . '>';
           

            // Loop through the query results
            if( $query->have_posts() ) {
                while( $query->have_posts() ) {
                    $query->the_post();

                    // Get post categories
                    $categories = get_the_category();
                    // Get post permalink
                    $permalink = esc_url(get_permalink());

                    // Start building the post item markup
                    $markup .= '<div class="bwdpg-post-item">';
                    $markup .= '<div class="bwdpg-post-image';
                    $markup .= $show_img_overlay ? esc_attr(' show-image-overlay') : '';
                    $markup .= '">';

                    // Featured image handling
                    if ($show_featured_image && has_post_thumbnail()) {
                        $featured_image = get_the_post_thumbnail();
                        $markup .= '<a href="' . $permalink . '">' . $featured_image . '</a>';
                    } else {
                        // Placeholder image if featured image is not available
                        $placeholder_image = plugin_dir_url(__FILE__) . 'assets/img/bwd-placeholder.jpg';
                        $markup .= '<a href="' . $permalink . '"><img src="' . $placeholder_image . '"></a>';
                    }
                    $markup .= '</div>'; 
                    // Post content section
                    $markup .= '<div class="bwdpg-post-content">';
                    $markup .= '<div class="bwdpg-post-dateTime">';
                     // Post date
                    if ( $show_date ) {
                        $published_date = get_the_date($date_format);
                        $markup .= '<div class="bwdpg-post-date">';
                        $markup .= '<span class="bwdpg-calender-icon"><span class="dashicons dashicons-calendar-alt"></span></span>'; 
                        $markup .= '<span class="bwdpg-date">' . esc_html($published_date) . '</span>';
                        $markup .= '</div>';
                    }
                    // Post estimate time
                    $markup .= '<div class="bwdpg-post-estimate">';
                    $words_per_minute = 200;
                    $post_word_count = str_word_count(strip_tags(get_the_content()));
                    $reading_time_minutes = ceil($post_word_count / $words_per_minute);
                    $markup .= '<span class="bwdpg-clock-icon"><span class="dashicons dashicons-clock"></span></span>';
                    $markup .= '<span class="bwdpg-estimate-time">' . esc_html($reading_time_minutes) . '</span>';
                    $markup .= '<span class="bwdpg-estimate-text">'. esc_html($read_time_suffix) .'</span>';

                    $markup .= '</div>';
                    $markup .= '</div>'; 
                    // Post title
                    if ($show_title) {
                        $title = wp_trim_words(get_the_title(), $title_length, '');
                        $title = rtrim($title, '.');
                        $markup .= '<' . esc_html($title_tag) . ' class="bwdpg-post-title"><a href="' . $permalink . '">' . esc_html($title) . '</a></' . esc_html($title_tag) . '>';
                    }
                    // Post excerpt
                    if ($show_description) {
                        $markup .= '<div class="exerpt">';
                        $excerpt = wp_trim_words(get_the_content(), $excerpt_length, '');
                        $markup .= '<p>' . esc_html($excerpt) . esc_html($tail_indicator) . '</p>';
                        $markup .= '</div>';
                    }
                    // Read more link
                    if ($show_read_more) {
                        $markup .= '<a href="' . $permalink . '" class="bwdpg-read-more">' . esc_html($read_more_text) . '</a>';
                    }
                    $markup .= '</div>';
                    // Post categories
                    if ($show_category && $categories) {
                        $markup .= '<span class="bwdpg-post-category">';
                        foreach ($categories as $category) {
                            $category_link = esc_url(get_category_link($category->term_id));
                            $category_name = esc_html($category->name);
                            $markup .= '<a href="' . $category_link . '">' . $category_name . '</a>';
                        }
                        $markup .= '</span>';
                    }
                    $markup .= '<div class="bwdpg-post-meta-box">';
                    // Author icon handling
                    if ($author_icon === 'gravatar') {
                        $author_avatar = get_avatar(get_the_author_meta('user_email'), 80);
                        $markup .= '<span class="bwdpg-author-icon">' . $author_avatar . '</span>';
                    } elseif ($author_icon === 'icon') {
                        $markup .= '<span class="bwdpg-author-icon"><span class="dashicons dashicons-admin-users"></span></span>';
                    }
                     // Post meta content
                    $markup .= '<div class="bwdpg-post-meta-content">';
                    $markup .= '<span class="bwdpg-author-prefix">' . esc_html($author_prefix) . '</span>';
                    // Author link
                    $author_link = esc_url(get_author_posts_url(get_the_author_meta('ID')));
                    $markup .= '<span class="post-author"><a href="' . $author_link . '">' . esc_html(get_the_author()) . '</a></span>';
                    $markup .= '</div>';
                    $markup .= '</div>';
                    $markup .= '</div>';
                }
                // Reset post data after the loop
                wp_reset_postdata();
            }
            $markup .= '</div>';
            // Pagination
            if ($show_pagination) {
                $pagination_args = array(
                    'total'      => $query->max_num_pages,
                    'prev_text' => '<svg enable-background="new 0 0 477.175 477.175" version="1.1" viewBox="0 0 477.18 477.18"><path d="m145.19 238.58 215.5-215.5c5.3-5.3 5.3-13.8 0-19.1s-13.8-5.3-19.1 0l-225.1 225.1c-5.3 5.3-5.3 13.8 0 19.1l225.1 225c2.6 2.6 6.1 4 9.5 4s6.9-1.3 9.5-4c5.3-5.3 5.3-13.8 0-19.1l-215.4-215.5z"></path></svg><span>'. esc_html($pagi_prev_text) .'</span>',
                    'next_text' => '<span>'. esc_html($pagi_next_text) .'</span><svg enable-background="new 0 0 477.175 477.175" version="1.1" viewBox="0 0 477.18 477.18"><path d="m360.73 229.08-225.1-225.1c-5.3-5.3-13.8-5.3-19.1 0s-5.3 13.8 0 19.1l215.5 215.5-215.5 215.5c-5.3 5.3-5.3 13.8 0 19.1 2.6 2.6 6.1 4 9.5 4s6.9-1.3 9.5-4l225.1-225.1c5.3-5.2 5.3-13.8 0.1-19z"></path></svg>',
                );

                $pagination = paginate_links($pagination_args);

                if ($pagination) {
                    $markup .= '<ul class="bwdpg-pagination">';
                    $markup .= '<li>' . $pagination . '</li>';
                    $markup .= '</ul>';
                }
            }
            $markup .= '</div>';


            // End output buffering and return the markup
            $markup .= ob_get_clean();

            return $markup;   
        }


    /**
     * Register Inline Style
     */
    function generate_inline_style_on_render_block($block_content, $block ) {

        if (isset($block['blockName']) && str_contains($block['blockName'], 'bwdpg/')) {
            if (isset($block['attrs']['blockStyle'])) {

                $style = $block['attrs']['blockStyle'];
                $handle = isset( $block['attrs']['uniqueId'] ) ? $block['attrs']['uniqueId'] : 'bwdpg-post-grid';

                // convert style array to string
                if ( is_array($style) ) {
                    $style = implode(' ', $style);
                }

                // minify style to remove extra space
                $style = preg_replace( '/\s+/', ' ', $style );

                wp_register_style(
                    $handle,
                    false,
                    [],
                    BWDPG_POST_GRID_VERSION
                );
                wp_enqueue_style( $handle );
                wp_add_inline_style( $handle, $style );

            }
        }
        return $block_content;
    }

}

new BWDPG_POST_GRID_BLOCKS_LOADER();